@@ -1,9 +1,9 @@
 package TAP::DOM;
-# git description: v0.99-1-ge8d5935
+# git description: v1.000-7-g37bf7ee
 
 our $AUTHORITY = 'cpan:SCHWIGON';
 # ABSTRACT: TAP as Document Object Model.
-$TAP::DOM::VERSION = '1.000';
+$TAP::DOM::VERSION = '1.001';
 use 5.006;
 use strict;
 use warnings;
@@ -36,6 +36,7 @@ our @tap_dom_args = (qw(ignore
                         dontignorelines
                         ignoreunknown
                         usebitsets
+                        sparse
                         disable_global_kv_data
                         put_dangling_kv_data_under_lazy_plan
                         document_data_prefix
@@ -47,6 +48,7 @@ our @tap_dom_args = (qw(ignore
                         lowercase_fieldnames
                         lowercase_fieldvalues
                         trim_fieldvalues
+                        normalize
                      ));
 
 use parent 'Exporter';
@@ -115,7 +117,7 @@ our $severity = {};
 #
 #          {type} {is_ok} {has_todo} {is_actual_ok} {has_skip} = $severity;
 #
-$severity->{plan}    {''}        {0}            {0}        {1} = 3; # ok_skip
+$severity->{plan}     {0}        {0}            {0}        {1} = 3; # ok_skip
 $severity->{test}     {1}        {0}            {0}        {0} = 1; # ok
 $severity->{test}     {1}        {1}            {1}        {0} = 2; # ok_todo
 $severity->{test}     {1}        {0}            {0}        {1} = 3; # ok_skip
@@ -152,6 +154,21 @@ sub _capture_group {
     my ($s, $n) = @_; substr($s, $-[$n], $+[$n] - $-[$n]);
 }
 
+sub normalize_tap_line {
+  my ($line) = @_;
+
+  return $line unless $line =~ m/^(not\s+)?(ok)\s+/;
+
+  $line =~ s{^(not\s)?\s*(ok)\s+(\d+\s)?\s*(-\s+)?\s*}{($1//'').$2.' '}e;
+  $line =~ s/['"]//g;
+  $line =~ s/\\(?!#\s*(todo|skip)\b)//gi;
+  $line =~ s/\s*(?<!\\)#\s*(todo|skip)\b/' # '.uc($1)/ei;
+  $line =~ s/^\s+//g;
+  $line =~ s/\s+$//g;
+
+  return $line;
+}
+
 # Optimize the TAP text before parsing it.
 sub preprocess_ignorelines {
     my %args = @_;
@@ -272,6 +289,7 @@ sub new {
         my $DONTIGNORELINES = $args{dontignorelines};
         my $IGNOREUNKNOWN = $args{ignoreunknown};
         my $USEBITSETS  = $args{usebitsets};
+        my $SPARSE  = $args{sparse};
         my $DISABLE_GLOBAL_KV_DATA  = $args{disable_global_kv_data};
         my $PUT_DANGLING_KV_DATA_UNDER_LAZY_PLAN  = $args{put_dangling_kv_data_under_lazy_plan};
         my $DOC_DATA_PREFIX = $args{document_data_prefix} || 'Test-';
@@ -280,11 +298,13 @@ sub new {
         my $LOWERCASE_FIELDVALUES = $args{lowercase_fieldvalues};
         my $TRIM_FIELDVALUES = $args{trim_fieldvalues};
         my $NOEMPTY_TAP = $args{noempty_tap};
+        my $NORMALIZE = $args{normalize};
         delete $args{ignore};
         delete $args{ignorelines};
         delete $args{dontignorelines};
         delete $args{ignoreunknown};
         delete $args{usebitsets};
+        delete $args{sparse};
         delete $args{disable_global_kv_data};
         delete $args{put_dangling_kv_data_under_lazy_plan};
         delete $args{document_data_prefix};
@@ -296,6 +316,7 @@ sub new {
         delete $args{lowercase_fieldnames};
         delete $args{lowercase_fieldvalues};
         delete $args{trim_fieldvalues};
+        delete $args{normalize};
 
         my $document_data_regex = qr/^#\s*$DOC_DATA_PREFIX([^:]+)\s*:\s*(.*)$/;
         my $document_data_ignore = defined($DOC_DATA_IGNORE) ? qr/$DOC_DATA_IGNORE/ : undef;
@@ -320,6 +341,7 @@ sub new {
                 foreach (qw(type raw as_string )) {
                         $entry->{$_} = $result->$_ unless $IGNORE{$_};
                 }
+                $entry->{normalized}  = $result->is_test ? normalize_tap_line($result->raw) : $result->raw;
 
                 if ($result->is_test) {
                         foreach (qw(directive explanation number description )) {
@@ -328,6 +350,9 @@ sub new {
                         foreach (qw(is_ok is_unplanned )) {
                                 if ($USEBITSETS) {
                                         $entry->{is_has} |= $result->$_ ? ${uc $_} : 0 unless $IGNORE{$_};
+                                } elsif ($SPARSE) {
+                                        # don't set 'false' fields at all in sparse mode
+                                        $entry->{$_} = 1 if $result->$_ and not $IGNORE{$_};
                                 } else {
                                         $entry->{$_} = $result->$_ ? 1 : 0 unless $IGNORE{$_};
                                 }
@@ -354,6 +379,9 @@ sub new {
                 foreach ((qw(has_skip has_todo))) {
                         if ($USEBITSETS) {
                                 $entry->{is_has} |= $result->$_ ? ${uc $_} : 0 unless $IGNORE{$_};
+                        } elsif ($SPARSE) {
+                                # don't set 'false' fields at all in sparse mode
+                                $entry->{$_} = 1 if $result->$_ and not $IGNORE{$_};
                         } else {
                                 $entry->{$_} = $result->$_ ? 1 : 0 unless $IGNORE{$_};
                         }
@@ -367,6 +395,9 @@ sub new {
                 {
                         if ($USEBITSETS) {
                                 $entry->{is_has} |= $result->$_ ? ${uc $_} : 0 unless $IGNORE{$_};
+                        } elsif ($SPARSE) {
+                                # don't set 'false' fields at all in sparse mode
+                                $entry->{$_} = 1 if $result->$_ and not $IGNORE{$_};
                         } else {
                                 $entry->{$_} = $result->$_ ? 1 : 0 unless $IGNORE{$_};
                         }
@@ -386,6 +417,9 @@ sub new {
                         my $is_actual_ok = ($result->has_todo && $result->is_actual_ok) ? 1 : 0;
                         if ($USEBITSETS) {
                                 $entry->{is_has} |= $is_actual_ok ? $IS_ACTUAL_OK : 0;
+                        } elsif ($SPARSE) {
+                                # don't set 'false' fields at all in sparse mode
+                                $entry->{is_actual_ok} = 1 if $is_actual_ok;
                         } else {
                                 $entry->{is_actual_ok} = $is_actual_ok;
                         }
@@ -434,10 +468,10 @@ sub new {
                   $count_tap_lines++;
                   $entry->{severity} = $severity
                     ->{$entry->{type}}
-                    ->{$entry->{is_ok}}
-                    ->{$entry->{has_todo}}
-                    ->{$entry->{is_actual_ok}}
-                    ->{$entry->{has_skip}};
+                    ->{$entry->{is_ok}||0}
+                    ->{$entry->{has_todo}||0}
+                    ->{$entry->{is_actual_ok}||0}
+                    ->{$entry->{has_skip}||0};
                 }
 
                 if ($entry->{is_pragma} or $entry->{is_unknown}) {
@@ -445,9 +479,9 @@ sub new {
                   if ($entry->{raw} =~ /^pragma\s+\+tapdom_error\s*$/) {
                     $found_pragma_tapdom_error=1;
                     $entry->{severity}   = 5;
-                    $entry->{is_unknown} = 0;
                     $entry->{is_pragma}  = 1;
                     $entry->{type}       = 'pragma';
+                    delete $entry->{is_unknown};
                   } else {
                     $entry->{severity} = 0;
                   }
@@ -469,41 +503,45 @@ sub new {
         if (!$count_tap_lines and !$found_pragma_tapdom_error and $NOEMPTY_TAP) {
           # pragma +tapdom_error
           my $error_entry = TAP::DOM::Entry->new(
-            'is_version'   => 0,
-            'is_plan'      => 0,
-            'is_test'      => 0,
-            'is_comment'   => 0,
-            'is_yaml'      => 0,
-            'is_unknown'   => 0,
-            'is_bailout'   => 0,
-            'is_actual_ok' => 0,
+            ($SPARSE ? () : (
+              'is_version'   => 0,
+              'is_plan'      => 0,
+              'is_test'      => 0,
+              'is_comment'   => 0,
+              'is_yaml'      => 0,
+              'is_unknown'   => 0,
+              'is_bailout'   => 0,
+              'is_actual_ok' => 0,
+              'has_todo'     => 0,
+              'has_skip'     => 0,
+            )),
             'is_pragma'    => 1,
             'type'         => 'pragma',
             'raw'          => 'pragma +tapdom_error',
             'as_string'    => 'pragma +tapdom_error',
             'severity'     => 5,
-            'has_todo'     => 0,
-            'has_skip'     => 0,
           );
           $error_entry->{is_has} = $IS_PRAGMA if $USEBITSETS;
           foreach (qw(raw type as_string explanation)) { delete $error_entry->{$_} if $IGNORE{$_} }
           # pragma +tapdom_error
           my $error_comment = TAP::DOM::Entry->new(
-            'is_version'   => 0,
-            'is_plan'      => 0,
-            'is_test'      => 0,
+            ($SPARSE ? () : (
+              'is_version'   => 0,
+              'is_plan'      => 0,
+              'is_test'      => 0,
+              'is_yaml'      => 0,
+              'is_unknown'   => 0,
+              'is_bailout'   => 0,
+              'is_actual_ok' => 0,
+              'is_pragma'    => 0,
+              'has_todo'     => 0,
+              'has_skip'     => 0,
+            )),
             'is_comment'   => 1,
-            'is_yaml'      => 0,
-            'is_unknown'   => 0,
-            'is_bailout'   => 0,
-            'is_actual_ok' => 0,
-            'is_pragma'    => 0,
             'type'         => 'comment',
             'raw'          => '# no tap lines',
             'as_string'    => '# no tap lines',
             'severity'     => 0,
-            'has_todo'     => 0,
-            'has_skip'     => 0,
           );
           $error_comment->{is_has} = $IS_COMMENT if $USEBITSETS;
           foreach (qw(raw type as_string explanation)) { delete $error_comment->{$_} if $IGNORE{$_} }
@@ -542,6 +580,7 @@ sub new {
           ignorelines                          => $IGNORELINES,
           dontignorelines                      => $DONTIGNORELINES,
           usebitsets                           => $USEBITSETS,
+          sparse                               => $SPARSE,
           disable_global_kv_data               => $DISABLE_GLOBAL_KV_DATA,
           put_dangling_kv_data_under_lazy_plan => $PUT_DANGLING_KV_DATA_UNDER_LAZY_PLAN,
           document_data_prefix                 => $DOC_DATA_PREFIX,
@@ -680,193 +719,6 @@ exploration tools", like L<Data::DPath|Data::DPath>.
 ``Reliable'' means that this structure is kind of an API that will not
 change, so your data tools can, well, rely on it.
 
-=head1 METHODS
-
-=head2 new
-
-Constructor which immediately triggers parsing the TAP via TAP::Parser
-and returns a big data structure containing the extracted results.
-
-=head3 Synopsis
-
- my $tap;
- {
-   local $/; open (TAP, '<', 't/some_tap.txt') or die;
-   $tap = <TAP>;
-   close TAP;
- }
- my $tapdata = TAP::DOM->new (
-   tap                                  => $tap
-   disable_global_kv_data               => 1,
-   put_dangling_kv_data_under_lazy_plan => 1,
-   ignorelines                          => '(## |# Test-mymeta_)',
-   dontignorelines                      => '# Test-mymeta_(tool1|tool2)_',
-   ignoreunknown                        => 1,
-   preprocess_ignorelines               => 1,
-   preprocess_tap                       => 1,
-   usebitsets                           => 0,
-   ignore                               => ['as_string'], # keep 'raw' which is the unmodified variant
-   document_data_prefix                 => '(MyApp|Test)-',
-   lowercase_fieldnames                 => 1,
-   trim_fieldvalues                     => 1,
- );
-
-=head3 Arguments
-
-=over 4
-
-=item ignore
-
-Arrayref of fieldnames not to contain in generated TAP::DOM. For
-example you can skip the C<as_string> field which is often a redundant
-variant of C<raw>.
-
-=item ignorelines
-
-A regular expression describing lines to ignore.
-
-Be careful to not screw up semantically relevant lines, like indented
-YAML data.
-
-The regex is internally prepended with a start-of-line C<^> anchor.
-
-=item dontignorelines (EXPERIMENTAL!)
-
-This is the whitelist of lines to B<not> being skipped when using the
-C<ignore> blacklist.
-
-The C<dontignorelines> feature is B<HIGHLY EXPERIMENTAL>, in
-particular in combination with C<preprocess_ignorelines>.
-
-Background: the preprocessing is done in a single regex operation for
-speed reasons, and to do that the C<dontignorelines> regex is turned
-into a I<zero-width negative-lookahead condition> and prepended before
-the C<ignorelines> condition into a combined regex.
-
-Without C<preprocess_ignorelines> it is a relatively harmless
-additional condition during TAP line processing.
-
-Survival tips:
-
-=over 2
-
-=item * have unit tests for your setup
-
-=item * do not use C<^> anchors neither in C<ignorelines> nor in
-C<dontignorelines> but rely on the implicitly prepended anchors.
-
-=item * write both C<ignorelines> and C<dontignorelines> completely
-describing from beginning of line (yet without the C<^> anchor).
-
-=item * do not use it but define C<ignorelines> instead with your own
-zero-width negative-lookaround conditions
-
-=item * know the zero-width negative look-around conditions of your
-use Perl version
-
-=back
-
-=item ignoreunknown
-
-By default non-TAP lines are still part of the TAP::DOM (with
-C<is_unknown=1> and most other entry fields set to C<undef>).
-
-If you mix a lot of non-TAP lines with actual TAP lines then
-this can lead to a huge TAP::DOM data structure.
-
-With this option set to 1 the C<unknown> lines are skipped.
-
-=item usebitsets
-
-Instead of having a lot of long boolean fields like
-
- has_skip => 1
- has_todo => 0
-
-you can encode all of them into a compact bitset
-
- is_has => $SOME_NUMERIC_REPRESENTATION
-
-This field must be evaluated later with bit-comparison operators.
-
-Originally meant as memory-saving mechanism it turned out not to be
-worth the hazzle.
-
-=item disable_global_kv_data
-
-Early TAP::DOM versions put all lines like
-
-  # Test-foo: bar
-
-into a global hash. Later these fields are placed as children under
-their parent C<ok>/C<not ok> line but kept globally for backwards
-compatibility. With this flag you can drop the redundant global hash.
-
-But see also C<put_dangling_kv_data_under_lazy_plan>.
-
-=item put_dangling_kv_data_under_lazy_plan
-
-This addresses the situation what to do in case a key/value field from
-a line
-
-  # Test-foo: bar
-
-appears without a parent C<ok>/C<not ok> line and the global kv_data
-hash is disabled. When this option is set it's placed under the plan
-as parent.
-
-=item document_data_prefix
-
-To interpret lines like
-
-  # Test-foo: bar
-
-the C<document_data_prefix> is by default set to C<Test-> so that a
-key/value field
-
-  foo => 'bar'
-
-is generated. However, you can have a regular expression to capture
-other or multiple different values as allowed prefixes.
-
-=item document_data_ignore
-
-This is another regex-based way to avoid generating particular
-fields. This regex is matched against the already extracted keys, and
-stops processing of this field for C<document_data> and C<kv_data>.
-
-=item lowercase_fieldnames
-
-If set to a true value all recognized fields are lowercased.
-
-=item lowercase_fieldvalues
-
-If set to a true value all recognized values are lowercased.
-
-=item trim_fieldvalues
-
-If set to a true value all field values are trimmed of trailing
-whitespace. Note that fields don't have leading whitespace as it's
-already consumed away after the fieldname separator colon C<:>.
-
-=back
-
-All other provided parameters are passed through to TAP::Parser, see
-sections "HOW TO STRIP DETAILS" and "USING BITSETS". Usually the
-options are just one of those:
-
-  tap => $some_tap_string
-
-or
-
-  source => $test_file
-
-But there are more, see L<TAP::Parser|TAP::Parser>.
-
-=head2 to_tap
-
-Called on a TAP::DOM object it returns a string that is TAP.
-
 =head1 STRUCTURE
 
 The data structure is basically a nested hash/array structure with
@@ -1060,7 +912,276 @@ into the line before under a key C<_children> which simply contains an
 array of those comment/yaml line elements.
 
 With this you can recognize where the diagnostic lines semantically
-belong.
+belong, i.e. to their I<parent> test line.
+
+=head1 METHODS
+
+=head2 new
+
+Constructor which immediately triggers parsing the TAP via TAP::Parser
+and returns a big data structure containing the extracted results.
+
+=head3 Synopsis
+
+ my $tap;
+ {
+   local $/; open (TAP, '<', 't/some_tap.txt') or die;
+   $tap = <TAP>;
+   close TAP;
+ }
+ my $tapdata = TAP::DOM->new (
+   tap                                  => $tap
+   disable_global_kv_data               => 1,
+   put_dangling_kv_data_under_lazy_plan => 1,
+   ignorelines                          => '(## |# Test-mymeta_)',
+   dontignorelines                      => '# Test-mymeta_(tool1|tool2)_',
+   ignoreunknown                        => 1,
+   preprocess_ignorelines               => 1,
+   preprocess_tap                       => 1,
+   usebitsets                           => 0,
+   sparse                               => 0,
+   ignore                               => ['as_string'], # keep 'raw' which is the unmodified variant
+   document_data_prefix                 => '(MyApp|Test)-',
+   lowercase_fieldnames                 => 1,
+   trim_fieldvalues                     => 1,
+ );
+
+=head3 Options
+
+=over 4
+
+=item ignore
+
+Arrayref of fieldnames not to contain in generated TAP::DOM. For
+example you can skip the C<as_string> field which is often a redundant
+variant of C<raw>.
+
+=item ignorelines
+
+A regular expression describing lines to ignore.
+
+Be careful to not screw up semantically relevant lines, like indented
+YAML data.
+
+The regex is internally prepended with a start-of-line C<^> anchor.
+
+=item dontignorelines (EXPERIMENTAL!)
+
+This is the whitelist of lines to B<not> being skipped when using the
+C<ignore> blacklist.
+
+The C<dontignorelines> feature is B<HIGHLY EXPERIMENTAL>, in
+particular in combination with C<preprocess_ignorelines>.
+
+Background: the preprocessing is done in a single regex operation for
+speed reasons, and to do that the C<dontignorelines> regex is turned
+into a I<zero-width negative-lookahead condition> and prepended before
+the C<ignorelines> condition into a combined regex.
+
+Without C<preprocess_ignorelines> it is a relatively harmless
+additional condition during TAP line processing.
+
+Survival tips:
+
+=over 2
+
+=item * have unit tests for your setup
+
+=item * do not use C<^> anchors neither in C<ignorelines> nor in
+C<dontignorelines> but rely on the implicitly prepended anchors.
+
+=item * write both C<ignorelines> and C<dontignorelines> completely
+describing from beginning of line (yet without the C<^> anchor).
+
+=item * do not use it but define C<ignorelines> instead with your own
+zero-width negative-lookaround conditions
+
+=item * know the zero-width negative look-around conditions of your
+use Perl version
+
+=back
+
+=item ignoreunknown
+
+By default non-TAP lines are still part of the TAP::DOM (with
+C<is_unknown=1> and most other entry fields set to C<undef>).
+
+If you mix a lot of non-TAP lines with actual TAP lines then
+this can lead to a huge TAP::DOM data structure.
+
+With this option set to 1 the C<unknown> lines are skipped.
+
+=item usebitsets
+
+Instead of having a lot of long boolean fields like
+
+ has_skip => 1
+ has_todo => 0
+
+you can encode all of them into a compact bitset
+
+ is_has => $SOME_NUMERIC_REPRESENTATION
+
+This field must be evaluated later with bit-comparison operators.
+
+Originally meant as memory-saving mechanism it turned out not to be
+worth the hazzle.
+
+=item sparse
+
+Only generate boolean fields if they are a true value. This results in
+a much more condensed data structure but also means you can't check on
+value C<0> but the absence of value C<1>.
+
+The option has no effect when C<usebitsets> is used.
+
+With the C<spare> option the actual line entries get restricted to
+only actually true values like this:
+
+  # ... (the initial part is same as above in chapter "STRUCTURE")
+  'lines' => [
+              {
+               'is_version'   => 1,
+               'severity'     => 0,
+               'raw'          => 'TAP version 13'
+               'as_string'    => 'TAP version 13',
+              },
+              {
+                'is_plan'      => 1,
+                'severity'     => 0,
+                'raw'          => '1..6'
+                'as_string'    => '1..6',
+              },
+              {
+                'is_ok'        => 1,
+                'is_test'      => 1,
+                'number'       => '1',
+                'severity'     => 1,
+                'type'         => 'test',
+                'raw'          => 'ok 1 - use Data::DPath;'
+                'as_string'    => 'ok 1 - use Data::DPath;',
+                'normalized'   => 'ok use Data::DPath;',
+                'description'  => '- use Data::DPath;',
+                'directive'    => '',
+                'explanation'  => '',
+                '_children'    => [
+                                   # ----- children are the subsequent comment/yaml lines -----
+                                   {
+                                     'is_yaml'      => 1,
+                                     'raw'          => '   ---
+     - name: \'Hash one\'
+       value: 1
+     - name: \'Hash two\'
+       value: 2
+   ...'
+                                     'as_string'    => '   ---
+     - name: \'Hash one\'
+       value: 1
+     - name: \'Hash two\'
+       value: 2
+   ...',
+                                     'data'         => [
+                                                        {
+                                                          'value' => '1',
+                                                          'name' => 'Hash one'
+                                                        },
+                                                        {
+                                                          'value' => '2',
+                                                          'name' => 'Hash two'
+                                                        }
+                                                       ],
+                                 }
+                               ],
+              },
+              {
+                'is_ok'        => 1,
+                'is_test'      => 1,
+                'explanation'  => '',
+                'number'       => '2',
+                'type'         => 'test',
+                'description'  => '- KEYs + PARENT',
+                'directive'    => '',
+                'severity'     => 1,
+                'raw'          => 'ok 2 - KEYs + PARENT'
+                'as_string'    => 'ok 2 - KEYs + PARENT',
+              },
+              # etc., see the rest in t/some_tap.dom ...
+             ],
+
+=item disable_global_kv_data
+
+Early TAP::DOM versions put all lines like
+
+  # Test-foo: bar
+
+into a global hash. Later these fields are placed as children under
+their parent C<ok>/C<not ok> line but kept globally for backwards
+compatibility. With this flag you can drop the redundant global hash.
+
+But see also C<put_dangling_kv_data_under_lazy_plan>.
+
+=item put_dangling_kv_data_under_lazy_plan
+
+This addresses the situation what to do in case a key/value field from
+a line
+
+  # Test-foo: bar
+
+appears without a parent C<ok>/C<not ok> line and the global kv_data
+hash is disabled. When this option is set it's placed under the plan
+as parent.
+
+=item document_data_prefix
+
+To interpret lines like
+
+  # Test-foo: bar
+
+the C<document_data_prefix> is by default set to C<Test-> so that a
+key/value field
+
+  foo => 'bar'
+
+is generated. However, you can have a regular expression to capture
+other or multiple different values as allowed prefixes.
+
+=item document_data_ignore
+
+This is another regex-based way to avoid generating particular
+fields. This regex is matched against the already extracted keys, and
+stops processing of this field for C<document_data> and C<kv_data>.
+
+=item lowercase_fieldnames
+
+If set to a true value all recognized fields are lowercased.
+
+=item lowercase_fieldvalues
+
+If set to a true value all recognized values are lowercased.
+
+=item trim_fieldvalues
+
+If set to a true value all field values are trimmed of trailing
+whitespace. Note that fields don't have leading whitespace as it's
+already consumed away after the fieldname separator colon C<:>.
+
+=back
+
+All other provided parameters are passed through to TAP::Parser, see
+sections "HOW TO STRIP DETAILS" and "USING BITSETS". Usually the
+options are just one of those:
+
+  tap => $some_tap_string
+
+or
+
+  source => $test_file
+
+But there are more, see L<TAP::Parser|TAP::Parser>.
+
+=head2 to_tap
+
+Called on a TAP::DOM object it returns a string that is TAP.
 
 =head1 HOW TO STRIP DETAILS
 
@@ -1104,7 +1225,7 @@ Use it like this:
 =head2 Strip unneccessary lines
 
 You can ignore complete lines from the input TAP as if they weren't
-existing by by setting a regular expression in C<ignorelines>. Of
+existing by setting a regular expression in C<ignorelines>. Of
 course you can break the TAP with this, so usually you only apply this
 to non-TAP lines or diagnostics you are not interested in.
 
@@ -1239,13 +1360,13 @@ By setting option C<lowercase_fieldnames> all field names (hash keys)
 in C<document_data> and C<kv_data> are set to lowercase. This is
 especially helpful to normalize different casing like
 
- # Test-Strange-Key: some value
- # Test-strange-key: some value
- # Test-STRANGE-KEY: some value
+ # Test-Strange-Key: Some Value
+ # Test-strange-key: Some Value
+ # Test-STRANGE-KEY: Some Value
 
 etc. all into
 
-  "strange-key" => "some value"
+  "strange-key" => "Some Value"
 
 =head2 Lowercase all key:value values
 
@@ -1253,13 +1374,13 @@ By setting option C<lowercase_fieldvalues> all field values in
 C<document_data> and C<kv_data> are set to lowercase. This is
 especially helpful to normalize different casing like
 
- # Test-strange-key: Some Value
- # Test-strange-key: Some value
- # Test-strange-key: SOME VALUE
+ # Test-Strange-Key: Some Value
+ # Test-Strange-Key: Some value
+ # Test-Strange-Key: SOME VALUE
 
 etc. all into
 
-  "strange-key" => "some value"
+  "Strange-Key" => "some value"
 
 B<Warning:> while the sister option C<lowercase_fieldnames> above is
 obviously helpful to keep the information more together, this
@@ -1270,7 +1391,7 @@ expressions, Elasticsearch, etc.
 
 =head2 Placing key:value pairs
 
-Normally a key:value pair C<{foo => bar}> from a line like
+Normally a key:value pair C<{foo =E<gt> bar}> from a line like
 
   # Test-foo: bar
 
@@ -1281,7 +1402,7 @@ If that's not the case then it is not clear where they belong. Early
 TAP::DOM versions had put them under a global entry C<document_data>.
 
 However this makes these entries inconsistently appear in different
-levels of the DOM. so you can suppress that old behaviour by setting
+levels of the DOM. Therefore you can suppress that old behaviour by setting
 C<disable_global_kv_data> to 1.
 
 However, with that option now, there can be lines that appear directly
@@ -1297,6 +1418,74 @@ Summary: for consistency it is suggested to set both options:
  disable_global_kv_data => 1,
  put_dangling_kv_data_under_lazy_plan => 1
 
+=head2 Normalize TAP line
+
+When the option C<normalize> is set to true an additional field
+L</normalized> is added which contains a, well, "normalized" (or
+"canonicalized") variant of the C<raw> line.
+
+Normalization rules:
+
+=over 4
+
+=item * line numbers, dashes, whitespace
+
+Different variants between the (not)ok part and the actual
+description, e.g. iteration number, leading dashes (like they were
+"invisible" in Larry Wall's original TAP handling), and leading and
+trailing whitespace are dropped:
+
+  not ok 12 - foo
+  not ok 12 foo
+  not ok - foo
+  not ok      foo
+
+get normalized to
+
+  not ok foo
+
+=item * quote characters
+
+Quote characters make it more difficult to transport the lines in
+typical string-based APIs like in JSON payload. We completely drop
+them (i.e., there is no replacement character):
+
+A line like
+
+  ok foo'bar"baz" drum'n'bass
+
+becomes
+
+  ok foobarbaz drumnbass
+
+=item * backslashes
+
+Backslashes are dropped, except before C<#TODO> / C<#SKIP> because
+there they are used by some TAP tools to force exactly these strings
+into the description, which is wrong imho but here we are...
+
+=item * directives (TODO, SKIP)
+
+Directives are normalized to uppercase with exactly one space left and
+right to C<#>, respectively, i.e. all these lines
+
+ not ok foo #todo my explanation
+ not ok foo # todo my explanation
+ not ok foo #TODO my explanation
+ not ok foo # TODO my explanation
+ not ok foo    #todo     my explanation
+
+become
+
+ not ok foo # TODO my explanation
+
+and the same with C<SKIP>.
+
+=back
+
+See also the L</normalize_tap_line> utility function for how to use
+this normalization outside of TAP::DOM.
+
 =head1 ACCESSORS AND UTILITY METHODS
 
 =head2 end_time
@@ -1355,6 +1544,36 @@ structure looks like this:
 
 =head2 version
 
+=head2 normalize_tap_line
+
+Utility function for inside and outside TAP::DOM usage.
+
+This function normalizes a TAP line, similar to what C<as_string()>
+does but it does it slightly different and does even more.
+
+It normalizes typical variants of formatting or numbering a TAP line
+into a common form. This can make searching TAP lines easier by not
+requiring regular expressions to express subtle variance. You would
+search by an also normalized TAP line in the stored normalized lines.
+
+The use of this functionality must be enabled as TAP::DOM option
+C<normalize>. Applications handling normalized TAP should also use
+this very function here to apply the same normalization.
+
+This method does not depend on the TAP parser but can be used
+independently on any string which is expected to be a test line
+(i.e. starting with C<ok> or C<not ok>), so it can be used from other
+applications, too. However, to ensure it makes sense you should ONLY
+apply it to lines that B<are> guaranteed to be such I<test> lines,
+which is B<not> as trivial as one might naively think. Inside TAP::DOM
+with the I<normalize> option they are only applied to lines where
+I<is_test> is true.
+
+See L</Normalize TAP line> about the applied modifications.
+
+Although the option C<normalize> itself is EXPERIMENTAL, the function
+normalize_tap_line itself can still make sense outside TAP::DOM usage.
+
 =head1 ADDITIONAL ATTRIBUTES
 
 TAP::DOM creates attributes beyond those from TAP::Parser, usually to
@@ -1412,6 +1631,17 @@ pragma doesn't interfere with C<test>/C<plan> lines you can use this
 to express an out-of-band error situation which would be lost
 otherwise. Read below for more.
 
+=head2 normalized (EXPERIMENTAL!)
+
+The C<normalized> field contains a normalized (or "canonicalized")
+variant of the C<raw> line. See L</Normalize TAP line> about the
+applied modifications.
+
+Please note, that the other fields, in particular C<description> and
+C<explanation> refer to the original C<raw> line as seen by the
+TAP::Parser, so they get out of sync. That's the main reason why this
+feature is B<EXPERIMENTAL>.
+
 =head1 TAP::DOM-SPECIFIC PRAGMAS
 
 Pragmas in TAP are meant to influence the behaviour of the TAP parser.
@@ -1429,9 +1659,9 @@ You can for instance append this pragma to the TAP document during
 post-processing to express an out-of-band error situation without
 interfering with the existing test lines and plan.
 
-Typical situations can be a an error from C<prove> or other TAP
-processor, and you want to ensure this problem does not get lost when
-storing the document in a database.
+Typical situations could be for instance errors from C<prove> or
+another TAP processor, and you want to ensure this problem does not
+get lost when storing the document in a database.
 
 Pragmas allow C<kv_data> like in C<test> and C<plan> lines, so you can
 transport additional error details like this:
@@ -1448,7 +1678,7 @@ Steffen Schwigon <ss5@renormalist.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2024 by Steffen Schwigon.
+This software is copyright (c) 2025 by Steffen Schwigon.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
